package javax.swing.plaf.basic;

import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.event.MenuKeyEvent;
import java.awt.Point;
import javax.swing.event.MenuDragMouseEvent;
import javax.swing.JMenuItem;
import java.beans.PropertyChangeEvent;
import javax.swing.event.ChangeEvent;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.Container;
import javax.swing.JPopupMenu;
import javax.swing.JMenuBar;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import javax.swing.plaf.UIResource;
import javax.swing.MenuSelectionManager;
import javax.swing.MenuElement;
import javax.swing.Timer;
import java.awt.Dimension;
import javax.swing.event.MenuKeyListener;
import javax.swing.event.MenuDragMouseListener;
import java.beans.PropertyChangeListener;
import javax.swing.event.MouseInputListener;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import sun.swing.DefaultLookup;
import javax.swing.UIManager;
import javax.swing.JMenu;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.InputMap;
import javax.swing.event.MenuListener;
import javax.swing.event.ChangeListener;

public class BasicMenuUI extends BasicMenuItemUI
{
    protected ChangeListener changeListener;
    protected MenuListener menuListener;
    private int lastMnemonic;
    private InputMap selectedWindowInputMap;
    private static final boolean TRACE = false;
    private static final boolean VERBOSE = false;
    private static final boolean DEBUG = false;
    private static boolean crossMenuMnemonic;
    
    public BasicMenuUI() {
        this.lastMnemonic = 0;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new BasicMenuUI();
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        BasicMenuItemUI.loadActionMap(lazyActionMap);
        lazyActionMap.put(new BasicMenuUI.Actions("selectMenu", null, true));
    }
    
    @Override
    protected void installDefaults() {
        super.installDefaults();
        this.updateDefaultBackgroundColor();
        ((JMenu)this.menuItem).setDelay(200);
        BasicMenuUI.crossMenuMnemonic = UIManager.getBoolean("Menu.crossMenuMnemonic");
    }
    
    @Override
    protected String getPropertyPrefix() {
        return "Menu";
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        if (this.changeListener == null) {
            this.changeListener = this.createChangeListener(this.menuItem);
        }
        if (this.changeListener != null) {
            this.menuItem.addChangeListener(this.changeListener);
        }
        if (this.menuListener == null) {
            this.menuListener = this.createMenuListener(this.menuItem);
        }
        if (this.menuListener != null) {
            ((JMenu)this.menuItem).addMenuListener(this.menuListener);
        }
    }
    
    @Override
    protected void installKeyboardActions() {
        super.installKeyboardActions();
        this.updateMnemonicBinding();
    }
    
    @Override
    void installLazyActionMap() {
        LazyActionMap.installLazyActionMap(this.menuItem, BasicMenuUI.class, this.getPropertyPrefix() + ".actionMap");
    }
    
    void updateMnemonicBinding() {
        int mnemonic = this.menuItem.getModel().getMnemonic();
        int[] array = (int[])(int[])DefaultLookup.get(this.menuItem, this, "Menu.shortcutKeys");
        if (array == null) {
            array = new int[] { 8 };
        }
        if (mnemonic == this.lastMnemonic) {
            return;
        }
        InputMap uiInputMap = SwingUtilities.getUIInputMap(this.menuItem, 2);
        if (this.lastMnemonic != 0 && uiInputMap != null) {
            for (int length = array.length, i = 0; i < length; ++i) {
                uiInputMap.remove(KeyStroke.getKeyStroke(this.lastMnemonic, array[i], false));
            }
        }
        if (mnemonic != 0) {
            if (uiInputMap == null) {
                uiInputMap = this.createInputMap(2);
                SwingUtilities.replaceUIInputMap(this.menuItem, 2, uiInputMap);
            }
            for (int length2 = array.length, j = 0; j < length2; ++j) {
                uiInputMap.put(KeyStroke.getKeyStroke(mnemonic, array[j], false), "selectMenu");
            }
        }
        this.lastMnemonic = mnemonic;
        return;
    }
    
    @Override
    protected void uninstallKeyboardActions() {
        super.uninstallKeyboardActions();
        this.lastMnemonic = 0;
    }
    
    @Override
    protected MouseInputListener createMouseInputListener(JComponent component) {
        return this.getHandler();
    }
    
    protected MenuListener createMenuListener(JComponent component) {
        return null;
    }
    
    protected ChangeListener createChangeListener(JComponent component) {
        return null;
    }
    
    @Override
    protected PropertyChangeListener createPropertyChangeListener(JComponent component) {
        return this.getHandler();
    }
    
    @Override
    BasicMenuItemUI.Handler getHandler() {
        if (this.handler == null) {
            this.handler = new BasicMenuUI.Handler();
        }
        return this.handler;
    }
    
    @Override
    protected void uninstallDefaults() {
        this.menuItem.setArmed(false);
        this.menuItem.setSelected(false);
        this.menuItem.resetKeyboardActions();
        super.uninstallDefaults();
    }
    
    @Override
    protected void uninstallListeners() {
        super.uninstallListeners();
        if (this.changeListener != null) {
            this.menuItem.removeChangeListener(this.changeListener);
        }
        if (this.menuListener != null) {
            ((JMenu)this.menuItem).removeMenuListener(this.menuListener);
        }
        this.changeListener = null;
        this.menuListener = null;
        this.handler = null;
    }
    
    @Override
    protected MenuDragMouseListener createMenuDragMouseListener(JComponent component) {
        return this.getHandler();
    }
    
    @Override
    protected MenuKeyListener createMenuKeyListener(JComponent component) {
        return (MenuKeyListener)this.getHandler();
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        if (((JMenu)this.menuItem).isTopLevelMenu()) {
            return new Dimension(component.getPreferredSize().width, 32767);
        }
        return null;
    }
    
    protected void setupPostTimer(JMenu menu) {
        Timer timer = new Timer(menu.getDelay(), new BasicMenuUI.Actions("selectMenu", menu, false));
        timer.setRepeats(false);
        timer.start();
    }
    
    private static void appendPath(MenuElement[] array, MenuElement menuElement) {
        MenuElement[] selectedPath = new MenuElement[array.length + 1];
        System.arraycopy(array, 0, selectedPath, 0, array.length);
        selectedPath[array.length] = menuElement;
        MenuSelectionManager.defaultManager().setSelectedPath(selectedPath);
    }
    
    private void updateDefaultBackgroundColor() {
        if (!UIManager.getBoolean("Menu.useMenuBarBackgroundForTopLevel")) {
            return;
        }
        JMenu menu = (JMenu)this.menuItem;
        if ((menu.getBackground() instanceof UIResource)) {
            if (menu.isTopLevelMenu()) {
                menu.setBackground(UIManager.getColor("MenuBar.background"));
            }
            else {
                menu.setBackground(UIManager.getColor(this.getPropertyPrefix() + ".background"));
            }
        }
        return;
    }
    
    static void access$200(MenuElement[] array, MenuElement menuElement) {
        appendPath(array, menuElement);
    }
    
    static void access$300(BasicMenuUI basicMenuUI) {
        basicMenuUI.updateDefaultBackgroundColor();
    }
    
    static {
        BasicMenuUI.crossMenuMnemonic = true;
    }
    
    private static class Actions extends UIAction
    {
        private static final String SELECT = "selectMenu";
        private JMenu menu;
        private boolean force;
        
        Actions(String name, JMenu menu, boolean force) {
            super(name);
            this.force = false;
            this.menu = menu;
            this.force = force;
        }
        
        private JMenu getMenu(ActionEvent actionEvent) {
            if ((actionEvent.getSource() instanceof JMenu)) {
                return (JMenu)actionEvent.getSource();
            }
            return this.menu;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JMenu menu = this.getMenu(actionEvent);
            if (!BasicMenuUI.crossMenuMnemonic) {
                JPopupMenu lastPopup = BasicPopupMenuUI.getLastPopup();
                if (lastPopup != null && lastPopup != menu.getParent()) {
                    return;
                }
            }
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            if (this.force) {
                Container parent = menu.getParent();
                if (parent != null && parent instanceof JMenuBar) {
                    MenuElement[] subElements = menu.getPopupMenu().getSubElements();
                    MenuElement[] selectedPath2;
                    if (subElements.length > 0) {
                        selectedPath2 = new MenuElement[] { (MenuElement)parent, menu, menu.getPopupMenu(), subElements[0] };
                    }
                    else {
                        selectedPath2 = new MenuElement[] { (MenuElement)parent, menu, menu.getPopupMenu() };
                    }
                    defaultManager.setSelectedPath(selectedPath2);
                }
            }
            else {
                MenuElement[] selectedPath = defaultManager.getSelectedPath();
                if (selectedPath.length > 0 && selectedPath[selectedPath.length - 1] == menu) {
                    BasicMenuUI.access$200(selectedPath, menu.getPopupMenu());
                }
            }
        }
        
        public boolean isEnabled(Object o) {
            return !(o instanceof JMenu) || ((JMenu)o).isEnabled();
        }
    }
    
    protected class MouseInputHandler implements MouseInputListener
    {
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mouseClicked(mouseEvent);
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mousePressed(mouseEvent);
        }
        
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mouseReleased(mouseEvent);
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mouseEntered(mouseEvent);
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mouseExited(mouseEvent);
        }
        
        @Override
        public void mouseDragged(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mouseDragged(mouseEvent);
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
            BasicMenuUI.this.getHandler().mouseMoved(mouseEvent);
        }
    }
    
    public class ChangeHandler implements ChangeListener
    {
        public JMenu menu;
        public BasicMenuUI ui;
        public boolean isSelected;
        public Component wasFocused;
        
        public ChangeHandler(JMenu menu, BasicMenuUI ui) {
            this.isSelected = false;
            this.menu = menu;
            this.ui = ui;
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
        }
    }
    
    private class Handler extends BasicMenuItemUI.Handler implements MenuKeyListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            if (propertyChangeEvent.getPropertyName() == "mnemonic") {
                BasicMenuUI.this.updateMnemonicBinding();
            }
            else {
                if (propertyChangeEvent.getPropertyName().equals("ancestor")) {
                    BasicMenuUI.access$300(BasicMenuUI.this);
                }
                super.propertyChange(propertyChangeEvent);
            }
        }
        
        @Override
        public void mouseClicked(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mousePressed(MouseEvent mouseEvent) {
            JMenu menu = (JMenu)BasicMenuUI.this.menuItem;
            if (!menu.isEnabled()) {
                return;
            }
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            if (menu.isTopLevelMenu()) {
                if (menu.isSelected() && menu.getPopupMenu().isShowing()) {
                    defaultManager.clearSelectedPath();
                }
                else {
                    Container parent = menu.getParent();
                    if (parent != null && parent instanceof JMenuBar) {
                        defaultManager.setSelectedPath(new MenuElement[] { (MenuElement)parent, menu });
                    }
                }
            }
            MenuElement[] selectedPath = defaultManager.getSelectedPath();
            if (selectedPath.length > 0 && selectedPath[selectedPath.length - 1] != menu.getPopupMenu()) {
                if (menu.isTopLevelMenu() || menu.getDelay() == 0) {
                    BasicMenuUI.access$200(selectedPath, menu.getPopupMenu());
                }
                else {
                    BasicMenuUI.this.setupPostTimer(menu);
                }
            }
            return;
        }
        
        @Override
        public void mouseReleased(MouseEvent event) {
            if (!((JMenuItem)BasicMenuUI.this.menuItem).isEnabled()) {
                return;
            }
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            defaultManager.processMouseEvent(event);
            if (!event.isConsumed()) {
                defaultManager.clearSelectedPath();
            }
            return;
        }
        
        @Override
        public void mouseEntered(MouseEvent mouseEvent) {
            JMenu menu = (JMenu)BasicMenuUI.this.menuItem;
            if (!menu.isEnabled() && !UIManager.getBoolean("MenuItem.disabledAreNavigable")) {
                return;
            }
            MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
            MenuElement[] selectedPath = defaultManager.getSelectedPath();
            if (!menu.isTopLevelMenu()) {
                if (selectedPath.length <= 0 || selectedPath[selectedPath.length - 1] != menu.getPopupMenu()) {
                    if (menu.getDelay() == 0) {
                        BasicMenuUI.access$200(BasicMenuUI.this.getPath(), menu.getPopupMenu());
                    }
                    else {
                        defaultManager.setSelectedPath(BasicMenuUI.this.getPath());
                        BasicMenuUI.this.setupPostTimer(menu);
                    }
                }
            }
            else if (selectedPath.length > 0 && selectedPath[0] == menu.getParent()) {
                MenuElement[] selectedPath2 = { (MenuElement)menu.getParent(), menu, null };
                if (BasicPopupMenuUI.getLastPopup() != null) {
                    selectedPath2[2] = menu.getPopupMenu();
                }
                defaultManager.setSelectedPath(selectedPath2);
            }
            return;
        }
        
        @Override
        public void mouseExited(MouseEvent mouseEvent) {
        }
        
        @Override
        public void mouseDragged(MouseEvent event) {
            if (!((JMenuItem)BasicMenuUI.this.menuItem).isEnabled()) {
                return;
            }
            MenuSelectionManager.defaultManager().processMouseEvent(event);
            return;
        }
        
        @Override
        public void mouseMoved(MouseEvent mouseEvent) {
        }
        
        @Override
        public void menuDragMouseEntered(MenuDragMouseEvent menuDragMouseEvent) {
        }
        
        @Override
        public void menuDragMouseDragged(MenuDragMouseEvent menuDragMouseEvent) {
            if (!BasicMenuUI.this.menuItem.isEnabled()) {
                return;
            }
            MenuSelectionManager menuSelectionManager = menuDragMouseEvent.getMenuSelectionManager();
            MenuElement[] path = menuDragMouseEvent.getPath();
            Point point = menuDragMouseEvent.getPoint();
            if (point.x >= 0 && point.x < BasicMenuUI.this.menuItem.getWidth() && point.y >= 0 && point.y < BasicMenuUI.this.menuItem.getHeight()) {
                JMenu menu = (JMenu)BasicMenuUI.this.menuItem;
                MenuElement[] selectedPath = menuSelectionManager.getSelectedPath();
                if (selectedPath.length <= 0 || selectedPath[selectedPath.length - 1] != menu.getPopupMenu()) {
                    if (menu.isTopLevelMenu() || menu.getDelay() == 0 || menuDragMouseEvent.getID() == 506) {
                        BasicMenuUI.access$200(path, menu.getPopupMenu());
                    }
                    else {
                        menuSelectionManager.setSelectedPath(path);
                        BasicMenuUI.this.setupPostTimer(menu);
                    }
                }
            }
            else if (menuDragMouseEvent.getID() == 502 && menuSelectionManager.componentForPoint(menuDragMouseEvent.getComponent(), menuDragMouseEvent.getPoint()) == null) {
                menuSelectionManager.clearSelectedPath();
            }
            return;
        }
        
        @Override
        public void menuDragMouseExited(MenuDragMouseEvent menuDragMouseEvent) {
        }
        
        @Override
        public void menuDragMouseReleased(MenuDragMouseEvent menuDragMouseEvent) {
        }
        
        @Override
        public void menuKeyTyped(MenuKeyEvent menuKeyEvent) {
            if (!BasicMenuUI.crossMenuMnemonic && BasicPopupMenuUI.getLastPopup() != null) {
                return;
            }
            if (BasicPopupMenuUI.getPopups().size() != 0) {
                return;
            }
            int lowerCase = Character.toLowerCase((char)BasicMenuUI.this.menuItem.getMnemonic());
            MenuElement[] path = menuKeyEvent.getPath();
            if (lowerCase == Character.toLowerCase(menuKeyEvent.getKeyChar())) {
                JPopupMenu popupMenu = ((JMenu)BasicMenuUI.this.menuItem).getPopupMenu();
                ArrayList list = new ArrayList(Arrays.asList(path));
                list.add(popupMenu);
                MenuElement enabledChild = BasicPopupMenuUI.findEnabledChild(popupMenu.getSubElements(), -1, true);
                if (enabledChild != null) {
                    list.add(enabledChild);
                }
                menuKeyEvent.getMenuSelectionManager().setSelectedPath((MenuElement[])(MenuElement[])list.toArray(new MenuElement[0]));
                menuKeyEvent.consume();
            }
            return;
        }
        
        @Override
        public void menuKeyPressed(MenuKeyEvent menuKeyEvent) {
        }
        
        @Override
        public void menuKeyReleased(MenuKeyEvent menuKeyEvent) {
        }
    }
}
